#//-----------------------------------------------
#//TUSB-S01PT用Pythonサンプルプログラム
#// 処理関数等
#//
#//2024_07_18　株式会社タートル工業
#//Copyright (C) 2024 Turtle Industry Co.,Ltd.
#//-----------------------------------------------

import tkinter as tk
import tkinter.ttk as ttk
import threading
from tkinter import messagebox
import time

def ShowInfo(str):
    """文字列を表示する"""
    messagebox.showinfo("",str)

def GetLFrame(master,title,relw,relh,relx,rely):
    """ラベル付きフレームを作成する
    
    arguments:
    master      ----    配置ペアレント
    title       ----    フレームのタイトル文字列
    relw        ----    表示幅(相対値)
    relh        ----    表示高(相対値)
    relx        ----    X座標相対位置(左端)
    rely        ----    Y座標相対位置(上端)

    returns:
    LabelFrame  ----    生成されたラベル付きフレームインスタンス
    """
    frame = tk.LabelFrame(master, text=title)
    frame.place(relwidth= relw,relheight=relh,relx=relx,rely=rely,anchor=tk.NW)
    return frame

def GetButton(master,text,command,relw,relx,rely):
    """ボタンを作成する
    
    arguments:
    master      ----    配置ペアレント
    text        ----    ボタンに表示する文字列
    command     ----    ボタンが押された時に呼び出されるハンドラ関数
    relw        ----    表示幅(相対値)
    relx        ----    X座標相対位置(左右中央)
    rely        ----    Y座標相対位置(上下中央)

    returns:
    Button      ----    生成されたボタンインスタンス
    """
    button = tk.Button(master,text=text,command=command)
    button.place(relwidth=relw,relx=relx,rely=rely,anchor=tk.CENTER)
    return button

def GetComboBox(master,value,relw,relx,rely,command=None):
    """コンボボックスを作成する
    
    arguments:
    master      ----    配置ペアレント
    value       ----    コンボボックスに設定するデータリスト(又はタプル)
    relw        ----    表示幅(相対値)
    relx        ----    X座標相対位置(左端)
    rely        ----    Y座標相対位置(上下中央)
    command     ----    選択値が変更した時に呼び出される関数(引数としてイベントが渡される)

    returns:
    Combobox    ----    生成されたコンボボックスインスタンス
    """
    combobox = ttk.Combobox(master,value=value,state="readonly")
    combobox.bind("<<ComboboxSelected>>",command)
    combobox.place(relwidth=relw,relx=relx,rely=rely,anchor=tk.W)
    combobox.current(0)
    return combobox

def GetLabel(master,text,relx,rely):
    """ラベルを作成する
    
    arguments:
    master      ----    配置ペアレント
    text        ----    表示する文字列
    relx        ----    X座標相対位置(左端)
    rely        ----    Y座標相対位置(上下中央)

    returns:
    Label       ----    生成されたラベルインスタンス
    """
    label = tk.Label(master,text=text)
    label.place(relx=relx,rely=rely,anchor=tk.W)
    return label

def GetLabelDisp(master,text,relw,relx,rely):
    """表示用ラベルを生成する
    
    arguments:
    master      ----    配置ペアレント
    text        ----    生成時に表示する文字列
    relw        ----    表示幅(相対値)
    relx        ----    X座標相対位置(左端)
    rely        ----    Y座標相対位置(上下中央)

    returns:
    Label       ----    生成された表示用ラベルのインスタンス
    """
    label = tk.Label(master,text=text,relief="groove")
    label.place(relwidth=relw,relx=relx,rely=rely,anchor=tk.W)
    return label

def GenListBox(master,relw,relh,relx,rely):
    """リストボックスを生成する
    
    arguments:
    master      ----    配置ペアレント
    relw        ----    表示幅(相対値)
    relh        ----    表示高さ(相対値)
    relx        ----    X座標相対位置(左端)
    rely        ----    Y座標相対位置(上端)

    returns:
    ListBox     ----    生成されたリストボックスのインスタンス
    """
    listbox = tk.Listbox(master)
    listbox.place(relwidth=relw,relheight=relh,relx=relx,rely=rely,anchor=tk.NW)
    return listbox

def GenList(p,s,initval):
    """list generator with value initialize
    
    Arguments:
    p           ----    number of elements on dimension 1
    s           ----    number of elements on dimension 1
    initval     ----    intialize value

    Return
    list        ----    generated list
    """
    if s == 0: return [initval for _ in range(p)]
    return [[initval for _ in range(s)] for _ in range(p)]

def GenTkVarList(p,s,cls,initval):
    """Tkinter Var list generator with value initialize
    
    arguments:
    p           ----    number of elements on dimension 1
    s           ----    number of elements on dimension 1
    cls         ----    tkinter variable class
    initval     ----    initial value
    
    returns:
    list        ----    generated list
    """
    if s == 0: return [cls(value=initval) for _ in range(p)]
    return [[cls(value=initval) for _ in range(s)] for _ in range(p)]

# タイマースレッド管理クラス ////////////////////////////////////
#厳密なインターバルではない。関数処理が終わったら指定時間待つだけ。
#from builtins import function
class intTT():
    """タイマースレッド管理クラス
    厳密なインターバル処理ではない。
    関数処理が終わったら指定時間待って再度実行しつづるループ。
    """
    thread :threading.Thread = None # スレッドクラスオブジェクト
    thread_end = None               # スレッド終了イベント
    call_func = None                # ループで実行する関数
    interval = None                 # ループの待ち時間

    def __init__(self,inttime,callf):
        """初期化"""
        self.interval = inttime # インターバルタイム
        self.call_func = callf  # 処理関数登録
        self.thread_end = threading.Event()
    
    def ThreadF(self):
        """スレッド関数(ループ)"""
        while not self.thread_end.wait(self.interval):
            self.call_func()

    def Start(self):
        """スレッド開始処理"""
        if self.thread == None:
            self.thread_end.clear()
            self.thread = threading.Thread(target=self.ThreadF)
            self.thread.start()
    
    def Stop(self):
        """スレッド停止処理"""
        if self.thread != None:
            self.thread_end.set()
            self.thread.join(0.1)
            self.thread = None 
